# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: feast/core/JobService.proto

from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from feast.core import DataSource_pb2 as feast_dot_core_dot_DataSource__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='feast/core/JobService.proto',
  package='feast.core',
  syntax='proto3',
  serialized_options=b'\n\020feast.proto.coreB\017JobServiceProtoZ3github.com/feast-dev/feast/sdk/go/protos/feast/core',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x1b\x66\x65\x61st/core/JobService.proto\x12\nfeast.core\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\x1b\x66\x65\x61st/core/DataSource.proto\"\xeb\x03\n\x03Job\x12\n\n\x02id\x18\x01 \x01(\t\x12!\n\x04type\x18\x02 \x01(\x0e\x32\x13.feast.core.JobType\x12%\n\x06status\x18\x03 \x01(\x0e\x32\x15.feast.core.JobStatus\x12\x0c\n\x04hash\x18\x04 \x01(\t\x12.\n\nstart_time\x18\x05 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x35\n\tretrieval\x18\x06 \x01(\x0b\x32 .feast.core.Job.RetrievalJobMetaH\x00\x12>\n\x0f\x62\x61tch_ingestion\x18\x07 \x01(\x0b\x32#.feast.core.Job.OfflineToOnlineMetaH\x00\x12>\n\x10stream_ingestion\x18\x08 \x01(\x0b\x32\".feast.core.Job.StreamToOnlineMetaH\x00\x12\x0f\n\x07log_uri\x18\t \x01(\t\x1a+\n\x10RetrievalJobMeta\x12\x17\n\x0foutput_location\x18\x01 \x01(\t\x1a)\n\x13OfflineToOnlineMeta\x12\x12\n\ntable_name\x18\x01 \x01(\t\x1a(\n\x12StreamToOnlineMeta\x12\x12\n\ntable_name\x18\x01 \x01(\tB\x06\n\x04meta\"\xac\x01\n\'StartOfflineToOnlineIngestionJobRequest\x12\x0f\n\x07project\x18\x01 \x01(\t\x12\x12\n\ntable_name\x18\x02 \x01(\t\x12.\n\nstart_date\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12,\n\x08\x65nd_date\x18\x04 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\"\x8f\x01\n(StartOfflineToOnlineIngestionJobResponse\x12\n\n\x02id\x18\x01 \x01(\t\x12\x32\n\x0ejob_start_time\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x12\n\ntable_name\x18\x03 \x01(\t\x12\x0f\n\x07log_uri\x18\x04 \x01(\t\"\xa4\x01\n\x1cGetHistoricalFeaturesRequest\x12\x14\n\x0c\x66\x65\x61ture_refs\x18\x01 \x03(\t\x12-\n\rentity_source\x18\x02 \x01(\x0b\x32\x16.feast.core.DataSource\x12\x0f\n\x07project\x18\x03 \x01(\t\x12\x17\n\x0foutput_location\x18\x04 \x01(\t\x12\x15\n\routput_format\x18\x05 \x01(\t\"\x89\x01\n\x1dGetHistoricalFeaturesResponse\x12\n\n\x02id\x18\x01 \x01(\t\x12\x17\n\x0foutput_file_uri\x18\x02 \x01(\t\x12\x32\n\x0ejob_start_time\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x0f\n\x07log_uri\x18\x04 \x01(\t\"M\n&StartStreamToOnlineIngestionJobRequest\x12\x0f\n\x07project\x18\x01 \x01(\t\x12\x12\n\ntable_name\x18\x02 \x01(\t\"\x8e\x01\n\'StartStreamToOnlineIngestionJobResponse\x12\n\n\x02id\x18\x01 \x01(\t\x12\x32\n\x0ejob_start_time\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x12\n\ntable_name\x18\x03 \x01(\t\x12\x0f\n\x07log_uri\x18\x04 \x01(\t\"-\n\x0fListJobsRequest\x12\x1a\n\x12include_terminated\x18\x01 \x01(\x08\"1\n\x10ListJobsResponse\x12\x1d\n\x04jobs\x18\x01 \x03(\x0b\x32\x0f.feast.core.Job\"\x1f\n\rGetJobRequest\x12\x0e\n\x06job_id\x18\x01 \x01(\t\".\n\x0eGetJobResponse\x12\x1c\n\x03job\x18\x01 \x01(\x0b\x32\x0f.feast.core.Job\"\"\n\x10\x43\x61ncelJobRequest\x12\x0e\n\x06job_id\x18\x01 \x01(\t\"\x13\n\x11\x43\x61ncelJobResponse*`\n\x07JobType\x12\x0f\n\x0bINVALID_JOB\x10\x00\x12\x17\n\x13\x42\x41TCH_INGESTION_JOB\x10\x01\x12\x18\n\x14STREAM_INGESTION_JOB\x10\x02\x12\x11\n\rRETRIEVAL_JOB\x10\x04*~\n\tJobStatus\x12\x16\n\x12JOB_STATUS_INVALID\x10\x00\x12\x16\n\x12JOB_STATUS_PENDING\x10\x01\x12\x16\n\x12JOB_STATUS_RUNNING\x10\x02\x12\x13\n\x0fJOB_STATUS_DONE\x10\x03\x12\x14\n\x10JOB_STATUS_ERROR\x10\x04\x32\xe9\x04\n\nJobService\x12\x8d\x01\n StartOfflineToOnlineIngestionJob\x12\x33.feast.core.StartOfflineToOnlineIngestionJobRequest\x1a\x34.feast.core.StartOfflineToOnlineIngestionJobResponse\x12l\n\x15GetHistoricalFeatures\x12(.feast.core.GetHistoricalFeaturesRequest\x1a).feast.core.GetHistoricalFeaturesResponse\x12\x8a\x01\n\x1fStartStreamToOnlineIngestionJob\x12\x32.feast.core.StartStreamToOnlineIngestionJobRequest\x1a\x33.feast.core.StartStreamToOnlineIngestionJobResponse\x12\x45\n\x08ListJobs\x12\x1b.feast.core.ListJobsRequest\x1a\x1c.feast.core.ListJobsResponse\x12H\n\tCancelJob\x12\x1c.feast.core.CancelJobRequest\x1a\x1d.feast.core.CancelJobResponse\x12?\n\x06GetJob\x12\x19.feast.core.GetJobRequest\x1a\x1a.feast.core.GetJobResponseBX\n\x10\x66\x65\x61st.proto.coreB\x0fJobServiceProtoZ3github.com/feast-dev/feast/sdk/go/protos/feast/coreb\x06proto3'
  ,
  dependencies=[google_dot_protobuf_dot_timestamp__pb2.DESCRIPTOR,feast_dot_core_dot_DataSource__pb2.DESCRIPTOR,])

_JOBTYPE = _descriptor.EnumDescriptor(
  name='JobType',
  full_name='feast.core.JobType',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='INVALID_JOB', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='BATCH_INGESTION_JOB', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='STREAM_INGESTION_JOB', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='RETRIEVAL_JOB', index=3, number=4,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=1687,
  serialized_end=1783,
)
_sym_db.RegisterEnumDescriptor(_JOBTYPE)

JobType = enum_type_wrapper.EnumTypeWrapper(_JOBTYPE)
_JOBSTATUS = _descriptor.EnumDescriptor(
  name='JobStatus',
  full_name='feast.core.JobStatus',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='JOB_STATUS_INVALID', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='JOB_STATUS_PENDING', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='JOB_STATUS_RUNNING', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='JOB_STATUS_DONE', index=3, number=3,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='JOB_STATUS_ERROR', index=4, number=4,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=1785,
  serialized_end=1911,
)
_sym_db.RegisterEnumDescriptor(_JOBSTATUS)

JobStatus = enum_type_wrapper.EnumTypeWrapper(_JOBSTATUS)
INVALID_JOB = 0
BATCH_INGESTION_JOB = 1
STREAM_INGESTION_JOB = 2
RETRIEVAL_JOB = 4
JOB_STATUS_INVALID = 0
JOB_STATUS_PENDING = 1
JOB_STATUS_RUNNING = 2
JOB_STATUS_DONE = 3
JOB_STATUS_ERROR = 4



_JOB_RETRIEVALJOBMETA = _descriptor.Descriptor(
  name='RetrievalJobMeta',
  full_name='feast.core.Job.RetrievalJobMeta',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='output_location', full_name='feast.core.Job.RetrievalJobMeta.output_location', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=461,
  serialized_end=504,
)

_JOB_OFFLINETOONLINEMETA = _descriptor.Descriptor(
  name='OfflineToOnlineMeta',
  full_name='feast.core.Job.OfflineToOnlineMeta',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='table_name', full_name='feast.core.Job.OfflineToOnlineMeta.table_name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=506,
  serialized_end=547,
)

_JOB_STREAMTOONLINEMETA = _descriptor.Descriptor(
  name='StreamToOnlineMeta',
  full_name='feast.core.Job.StreamToOnlineMeta',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='table_name', full_name='feast.core.Job.StreamToOnlineMeta.table_name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=549,
  serialized_end=589,
)

_JOB = _descriptor.Descriptor(
  name='Job',
  full_name='feast.core.Job',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='feast.core.Job.id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='type', full_name='feast.core.Job.type', index=1,
      number=2, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='status', full_name='feast.core.Job.status', index=2,
      number=3, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='hash', full_name='feast.core.Job.hash', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='start_time', full_name='feast.core.Job.start_time', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='retrieval', full_name='feast.core.Job.retrieval', index=5,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='batch_ingestion', full_name='feast.core.Job.batch_ingestion', index=6,
      number=7, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='stream_ingestion', full_name='feast.core.Job.stream_ingestion', index=7,
      number=8, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='log_uri', full_name='feast.core.Job.log_uri', index=8,
      number=9, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_JOB_RETRIEVALJOBMETA, _JOB_OFFLINETOONLINEMETA, _JOB_STREAMTOONLINEMETA, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='meta', full_name='feast.core.Job.meta',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=106,
  serialized_end=597,
)


_STARTOFFLINETOONLINEINGESTIONJOBREQUEST = _descriptor.Descriptor(
  name='StartOfflineToOnlineIngestionJobRequest',
  full_name='feast.core.StartOfflineToOnlineIngestionJobRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='project', full_name='feast.core.StartOfflineToOnlineIngestionJobRequest.project', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='table_name', full_name='feast.core.StartOfflineToOnlineIngestionJobRequest.table_name', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='start_date', full_name='feast.core.StartOfflineToOnlineIngestionJobRequest.start_date', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='end_date', full_name='feast.core.StartOfflineToOnlineIngestionJobRequest.end_date', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=600,
  serialized_end=772,
)


_STARTOFFLINETOONLINEINGESTIONJOBRESPONSE = _descriptor.Descriptor(
  name='StartOfflineToOnlineIngestionJobResponse',
  full_name='feast.core.StartOfflineToOnlineIngestionJobResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='feast.core.StartOfflineToOnlineIngestionJobResponse.id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='job_start_time', full_name='feast.core.StartOfflineToOnlineIngestionJobResponse.job_start_time', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='table_name', full_name='feast.core.StartOfflineToOnlineIngestionJobResponse.table_name', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='log_uri', full_name='feast.core.StartOfflineToOnlineIngestionJobResponse.log_uri', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=775,
  serialized_end=918,
)


_GETHISTORICALFEATURESREQUEST = _descriptor.Descriptor(
  name='GetHistoricalFeaturesRequest',
  full_name='feast.core.GetHistoricalFeaturesRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='feature_refs', full_name='feast.core.GetHistoricalFeaturesRequest.feature_refs', index=0,
      number=1, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='entity_source', full_name='feast.core.GetHistoricalFeaturesRequest.entity_source', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='project', full_name='feast.core.GetHistoricalFeaturesRequest.project', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='output_location', full_name='feast.core.GetHistoricalFeaturesRequest.output_location', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='output_format', full_name='feast.core.GetHistoricalFeaturesRequest.output_format', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=921,
  serialized_end=1085,
)


_GETHISTORICALFEATURESRESPONSE = _descriptor.Descriptor(
  name='GetHistoricalFeaturesResponse',
  full_name='feast.core.GetHistoricalFeaturesResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='feast.core.GetHistoricalFeaturesResponse.id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='output_file_uri', full_name='feast.core.GetHistoricalFeaturesResponse.output_file_uri', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='job_start_time', full_name='feast.core.GetHistoricalFeaturesResponse.job_start_time', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='log_uri', full_name='feast.core.GetHistoricalFeaturesResponse.log_uri', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1088,
  serialized_end=1225,
)


_STARTSTREAMTOONLINEINGESTIONJOBREQUEST = _descriptor.Descriptor(
  name='StartStreamToOnlineIngestionJobRequest',
  full_name='feast.core.StartStreamToOnlineIngestionJobRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='project', full_name='feast.core.StartStreamToOnlineIngestionJobRequest.project', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='table_name', full_name='feast.core.StartStreamToOnlineIngestionJobRequest.table_name', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1227,
  serialized_end=1304,
)


_STARTSTREAMTOONLINEINGESTIONJOBRESPONSE = _descriptor.Descriptor(
  name='StartStreamToOnlineIngestionJobResponse',
  full_name='feast.core.StartStreamToOnlineIngestionJobResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='feast.core.StartStreamToOnlineIngestionJobResponse.id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='job_start_time', full_name='feast.core.StartStreamToOnlineIngestionJobResponse.job_start_time', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='table_name', full_name='feast.core.StartStreamToOnlineIngestionJobResponse.table_name', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='log_uri', full_name='feast.core.StartStreamToOnlineIngestionJobResponse.log_uri', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1307,
  serialized_end=1449,
)


_LISTJOBSREQUEST = _descriptor.Descriptor(
  name='ListJobsRequest',
  full_name='feast.core.ListJobsRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='include_terminated', full_name='feast.core.ListJobsRequest.include_terminated', index=0,
      number=1, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1451,
  serialized_end=1496,
)


_LISTJOBSRESPONSE = _descriptor.Descriptor(
  name='ListJobsResponse',
  full_name='feast.core.ListJobsResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='jobs', full_name='feast.core.ListJobsResponse.jobs', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1498,
  serialized_end=1547,
)


_GETJOBREQUEST = _descriptor.Descriptor(
  name='GetJobRequest',
  full_name='feast.core.GetJobRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='job_id', full_name='feast.core.GetJobRequest.job_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1549,
  serialized_end=1580,
)


_GETJOBRESPONSE = _descriptor.Descriptor(
  name='GetJobResponse',
  full_name='feast.core.GetJobResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='job', full_name='feast.core.GetJobResponse.job', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1582,
  serialized_end=1628,
)


_CANCELJOBREQUEST = _descriptor.Descriptor(
  name='CancelJobRequest',
  full_name='feast.core.CancelJobRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='job_id', full_name='feast.core.CancelJobRequest.job_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1630,
  serialized_end=1664,
)


_CANCELJOBRESPONSE = _descriptor.Descriptor(
  name='CancelJobResponse',
  full_name='feast.core.CancelJobResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1666,
  serialized_end=1685,
)

_JOB_RETRIEVALJOBMETA.containing_type = _JOB
_JOB_OFFLINETOONLINEMETA.containing_type = _JOB
_JOB_STREAMTOONLINEMETA.containing_type = _JOB
_JOB.fields_by_name['type'].enum_type = _JOBTYPE
_JOB.fields_by_name['status'].enum_type = _JOBSTATUS
_JOB.fields_by_name['start_time'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_JOB.fields_by_name['retrieval'].message_type = _JOB_RETRIEVALJOBMETA
_JOB.fields_by_name['batch_ingestion'].message_type = _JOB_OFFLINETOONLINEMETA
_JOB.fields_by_name['stream_ingestion'].message_type = _JOB_STREAMTOONLINEMETA
_JOB.oneofs_by_name['meta'].fields.append(
  _JOB.fields_by_name['retrieval'])
_JOB.fields_by_name['retrieval'].containing_oneof = _JOB.oneofs_by_name['meta']
_JOB.oneofs_by_name['meta'].fields.append(
  _JOB.fields_by_name['batch_ingestion'])
_JOB.fields_by_name['batch_ingestion'].containing_oneof = _JOB.oneofs_by_name['meta']
_JOB.oneofs_by_name['meta'].fields.append(
  _JOB.fields_by_name['stream_ingestion'])
_JOB.fields_by_name['stream_ingestion'].containing_oneof = _JOB.oneofs_by_name['meta']
_STARTOFFLINETOONLINEINGESTIONJOBREQUEST.fields_by_name['start_date'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_STARTOFFLINETOONLINEINGESTIONJOBREQUEST.fields_by_name['end_date'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_STARTOFFLINETOONLINEINGESTIONJOBRESPONSE.fields_by_name['job_start_time'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_GETHISTORICALFEATURESREQUEST.fields_by_name['entity_source'].message_type = feast_dot_core_dot_DataSource__pb2._DATASOURCE
_GETHISTORICALFEATURESRESPONSE.fields_by_name['job_start_time'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_STARTSTREAMTOONLINEINGESTIONJOBRESPONSE.fields_by_name['job_start_time'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_LISTJOBSRESPONSE.fields_by_name['jobs'].message_type = _JOB
_GETJOBRESPONSE.fields_by_name['job'].message_type = _JOB
DESCRIPTOR.message_types_by_name['Job'] = _JOB
DESCRIPTOR.message_types_by_name['StartOfflineToOnlineIngestionJobRequest'] = _STARTOFFLINETOONLINEINGESTIONJOBREQUEST
DESCRIPTOR.message_types_by_name['StartOfflineToOnlineIngestionJobResponse'] = _STARTOFFLINETOONLINEINGESTIONJOBRESPONSE
DESCRIPTOR.message_types_by_name['GetHistoricalFeaturesRequest'] = _GETHISTORICALFEATURESREQUEST
DESCRIPTOR.message_types_by_name['GetHistoricalFeaturesResponse'] = _GETHISTORICALFEATURESRESPONSE
DESCRIPTOR.message_types_by_name['StartStreamToOnlineIngestionJobRequest'] = _STARTSTREAMTOONLINEINGESTIONJOBREQUEST
DESCRIPTOR.message_types_by_name['StartStreamToOnlineIngestionJobResponse'] = _STARTSTREAMTOONLINEINGESTIONJOBRESPONSE
DESCRIPTOR.message_types_by_name['ListJobsRequest'] = _LISTJOBSREQUEST
DESCRIPTOR.message_types_by_name['ListJobsResponse'] = _LISTJOBSRESPONSE
DESCRIPTOR.message_types_by_name['GetJobRequest'] = _GETJOBREQUEST
DESCRIPTOR.message_types_by_name['GetJobResponse'] = _GETJOBRESPONSE
DESCRIPTOR.message_types_by_name['CancelJobRequest'] = _CANCELJOBREQUEST
DESCRIPTOR.message_types_by_name['CancelJobResponse'] = _CANCELJOBRESPONSE
DESCRIPTOR.enum_types_by_name['JobType'] = _JOBTYPE
DESCRIPTOR.enum_types_by_name['JobStatus'] = _JOBSTATUS
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Job = _reflection.GeneratedProtocolMessageType('Job', (_message.Message,), {

  'RetrievalJobMeta' : _reflection.GeneratedProtocolMessageType('RetrievalJobMeta', (_message.Message,), {
    'DESCRIPTOR' : _JOB_RETRIEVALJOBMETA,
    '__module__' : 'feast.core.JobService_pb2'
    # @@protoc_insertion_point(class_scope:feast.core.Job.RetrievalJobMeta)
    })
  ,

  'OfflineToOnlineMeta' : _reflection.GeneratedProtocolMessageType('OfflineToOnlineMeta', (_message.Message,), {
    'DESCRIPTOR' : _JOB_OFFLINETOONLINEMETA,
    '__module__' : 'feast.core.JobService_pb2'
    # @@protoc_insertion_point(class_scope:feast.core.Job.OfflineToOnlineMeta)
    })
  ,

  'StreamToOnlineMeta' : _reflection.GeneratedProtocolMessageType('StreamToOnlineMeta', (_message.Message,), {
    'DESCRIPTOR' : _JOB_STREAMTOONLINEMETA,
    '__module__' : 'feast.core.JobService_pb2'
    # @@protoc_insertion_point(class_scope:feast.core.Job.StreamToOnlineMeta)
    })
  ,
  'DESCRIPTOR' : _JOB,
  '__module__' : 'feast.core.JobService_pb2'
  # @@protoc_insertion_point(class_scope:feast.core.Job)
  })
_sym_db.RegisterMessage(Job)
_sym_db.RegisterMessage(Job.RetrievalJobMeta)
_sym_db.RegisterMessage(Job.OfflineToOnlineMeta)
_sym_db.RegisterMessage(Job.StreamToOnlineMeta)

StartOfflineToOnlineIngestionJobRequest = _reflection.GeneratedProtocolMessageType('StartOfflineToOnlineIngestionJobRequest', (_message.Message,), {
  'DESCRIPTOR' : _STARTOFFLINETOONLINEINGESTIONJOBREQUEST,
  '__module__' : 'feast.core.JobService_pb2'
  # @@protoc_insertion_point(class_scope:feast.core.StartOfflineToOnlineIngestionJobRequest)
  })
_sym_db.RegisterMessage(StartOfflineToOnlineIngestionJobRequest)

StartOfflineToOnlineIngestionJobResponse = _reflection.GeneratedProtocolMessageType('StartOfflineToOnlineIngestionJobResponse', (_message.Message,), {
  'DESCRIPTOR' : _STARTOFFLINETOONLINEINGESTIONJOBRESPONSE,
  '__module__' : 'feast.core.JobService_pb2'
  # @@protoc_insertion_point(class_scope:feast.core.StartOfflineToOnlineIngestionJobResponse)
  })
_sym_db.RegisterMessage(StartOfflineToOnlineIngestionJobResponse)

GetHistoricalFeaturesRequest = _reflection.GeneratedProtocolMessageType('GetHistoricalFeaturesRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETHISTORICALFEATURESREQUEST,
  '__module__' : 'feast.core.JobService_pb2'
  # @@protoc_insertion_point(class_scope:feast.core.GetHistoricalFeaturesRequest)
  })
_sym_db.RegisterMessage(GetHistoricalFeaturesRequest)

GetHistoricalFeaturesResponse = _reflection.GeneratedProtocolMessageType('GetHistoricalFeaturesResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETHISTORICALFEATURESRESPONSE,
  '__module__' : 'feast.core.JobService_pb2'
  # @@protoc_insertion_point(class_scope:feast.core.GetHistoricalFeaturesResponse)
  })
_sym_db.RegisterMessage(GetHistoricalFeaturesResponse)

StartStreamToOnlineIngestionJobRequest = _reflection.GeneratedProtocolMessageType('StartStreamToOnlineIngestionJobRequest', (_message.Message,), {
  'DESCRIPTOR' : _STARTSTREAMTOONLINEINGESTIONJOBREQUEST,
  '__module__' : 'feast.core.JobService_pb2'
  # @@protoc_insertion_point(class_scope:feast.core.StartStreamToOnlineIngestionJobRequest)
  })
_sym_db.RegisterMessage(StartStreamToOnlineIngestionJobRequest)

StartStreamToOnlineIngestionJobResponse = _reflection.GeneratedProtocolMessageType('StartStreamToOnlineIngestionJobResponse', (_message.Message,), {
  'DESCRIPTOR' : _STARTSTREAMTOONLINEINGESTIONJOBRESPONSE,
  '__module__' : 'feast.core.JobService_pb2'
  # @@protoc_insertion_point(class_scope:feast.core.StartStreamToOnlineIngestionJobResponse)
  })
_sym_db.RegisterMessage(StartStreamToOnlineIngestionJobResponse)

ListJobsRequest = _reflection.GeneratedProtocolMessageType('ListJobsRequest', (_message.Message,), {
  'DESCRIPTOR' : _LISTJOBSREQUEST,
  '__module__' : 'feast.core.JobService_pb2'
  # @@protoc_insertion_point(class_scope:feast.core.ListJobsRequest)
  })
_sym_db.RegisterMessage(ListJobsRequest)

ListJobsResponse = _reflection.GeneratedProtocolMessageType('ListJobsResponse', (_message.Message,), {
  'DESCRIPTOR' : _LISTJOBSRESPONSE,
  '__module__' : 'feast.core.JobService_pb2'
  # @@protoc_insertion_point(class_scope:feast.core.ListJobsResponse)
  })
_sym_db.RegisterMessage(ListJobsResponse)

GetJobRequest = _reflection.GeneratedProtocolMessageType('GetJobRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETJOBREQUEST,
  '__module__' : 'feast.core.JobService_pb2'
  # @@protoc_insertion_point(class_scope:feast.core.GetJobRequest)
  })
_sym_db.RegisterMessage(GetJobRequest)

GetJobResponse = _reflection.GeneratedProtocolMessageType('GetJobResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETJOBRESPONSE,
  '__module__' : 'feast.core.JobService_pb2'
  # @@protoc_insertion_point(class_scope:feast.core.GetJobResponse)
  })
_sym_db.RegisterMessage(GetJobResponse)

CancelJobRequest = _reflection.GeneratedProtocolMessageType('CancelJobRequest', (_message.Message,), {
  'DESCRIPTOR' : _CANCELJOBREQUEST,
  '__module__' : 'feast.core.JobService_pb2'
  # @@protoc_insertion_point(class_scope:feast.core.CancelJobRequest)
  })
_sym_db.RegisterMessage(CancelJobRequest)

CancelJobResponse = _reflection.GeneratedProtocolMessageType('CancelJobResponse', (_message.Message,), {
  'DESCRIPTOR' : _CANCELJOBRESPONSE,
  '__module__' : 'feast.core.JobService_pb2'
  # @@protoc_insertion_point(class_scope:feast.core.CancelJobResponse)
  })
_sym_db.RegisterMessage(CancelJobResponse)


DESCRIPTOR._options = None

_JOBSERVICE = _descriptor.ServiceDescriptor(
  name='JobService',
  full_name='feast.core.JobService',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_start=1914,
  serialized_end=2531,
  methods=[
  _descriptor.MethodDescriptor(
    name='StartOfflineToOnlineIngestionJob',
    full_name='feast.core.JobService.StartOfflineToOnlineIngestionJob',
    index=0,
    containing_service=None,
    input_type=_STARTOFFLINETOONLINEINGESTIONJOBREQUEST,
    output_type=_STARTOFFLINETOONLINEINGESTIONJOBRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetHistoricalFeatures',
    full_name='feast.core.JobService.GetHistoricalFeatures',
    index=1,
    containing_service=None,
    input_type=_GETHISTORICALFEATURESREQUEST,
    output_type=_GETHISTORICALFEATURESRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='StartStreamToOnlineIngestionJob',
    full_name='feast.core.JobService.StartStreamToOnlineIngestionJob',
    index=2,
    containing_service=None,
    input_type=_STARTSTREAMTOONLINEINGESTIONJOBREQUEST,
    output_type=_STARTSTREAMTOONLINEINGESTIONJOBRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='ListJobs',
    full_name='feast.core.JobService.ListJobs',
    index=3,
    containing_service=None,
    input_type=_LISTJOBSREQUEST,
    output_type=_LISTJOBSRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='CancelJob',
    full_name='feast.core.JobService.CancelJob',
    index=4,
    containing_service=None,
    input_type=_CANCELJOBREQUEST,
    output_type=_CANCELJOBRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetJob',
    full_name='feast.core.JobService.GetJob',
    index=5,
    containing_service=None,
    input_type=_GETJOBREQUEST,
    output_type=_GETJOBRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
])
_sym_db.RegisterServiceDescriptor(_JOBSERVICE)

DESCRIPTOR.services_by_name['JobService'] = _JOBSERVICE

# @@protoc_insertion_point(module_scope)
